﻿using ProjectTrolleybus.Drawnings;
namespace ProjectTrolleybus.CollectionGenericObjects;

/// <summary>
/// Интерфейс описания действий для набора хранимых объектов
/// </summary>
/// <typeparam name="T">Параметр: ограничение - ссылочный тип</typeparam>
public interface ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
    /// Кол-во объектов в коллекци
    /// </summary>
    int Count { get; }

    /// <summary>
    /// Установка максимального кол-ва объектов
    /// </summary>
    int MaxCount { get; set; }

    /// <summary>
    /// Добавление объекта в коллецию
    /// </summary>
    /// <param name="obj"></param>
    /// <returns></returns>
    int Insert(T obj, IEqualityComparer<DrawningTrolleyB?>? comparer = null);

    /// <summary>
    /// Добавление объекта в коллекцию
    /// </summary>
    /// <param name="obj">Добавляемый объект</param>
    /// <returns>true - вставка прошла удачно, false - вставка не удалась</returns>
    int Insert(T obj, int position, IEqualityComparer<DrawningTrolleyB?>? comparer = null);

    /// <summary>
    /// Удаление объекта из коллекции с конкретной позиции 
    /// </summary>
    /// <param name="position">Позиция</param>
    /// <returns>true - удаление прошло удачно, false - удаление не удалось</returns>
    T? Remove(int position);
    
    /// <summary>
    /// Получение объекта и з позиции
    /// </summary>
    /// <param name="position">Позиция</param>
    /// <returns>Объект</returns>
    T? Get(int position);

    /// <summary>
    /// Получение типа коллекции
    /// </summary>
    CollectionType GetCollectionType { get; }

    /// <summary>
    /// Получение объектов коллекции по одному
    /// </summary>
    /// <returns>Поэлементный вывод элементов коллекции</returns>
    IEnumerable<T?> GetItems();

    /// <summary>
    /// Сортировка коллекции
    /// </summary>
    /// <param name="comparer">Сравниватель объектов</param>
    void CollectionSort(IComparer<T?> comparer);

}
